﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

using Nomad.Net.PacketCapture.Interop;
using Nomad.Net.PacketCapture.Interop.WinPCap;

namespace basic_dump
{
    class Program
    {

        public static readonly IntPtr nil = IntPtr.Zero;
        static void Main(string[] args)
        {

            IntPtr head = nil;
            IntPtr current = nil;
            IntPtr capture = nil;
            pcap_if adapter;
            int adapterNumber = 0;
            StringBuilder errbuf = new StringBuilder(WinPCapConstants.PCAP_ERRBUF_SIZE);
            
            //get device list
            if (WinPCapDriver.pcap_findalldevs(ref head, errbuf) < 0)
            {
                Console.WriteLine("Error in pcap_findalldevs");
                return;
            }

            //print list for user to select from
            for (current = head; current != nil; current = adapter.next)
            {

                adapter = current.AsStruct<pcap_if>();
                Console.WriteLine("{0}. {1}", ++adapterNumber, adapter.description);

            }

            if (adapterNumber == 0)
            {
                Console.WriteLine("No adapters found! Makesure WinPCap is installed.");
                return;
            }

            //get the user to select a capture interface
            Console.Write("Enter Captuer Interface Number[1 - {0}]: ", adapterNumber);
            int selected;
            string userEntry = Console.ReadLine();

            if ((!int.TryParse(userEntry, out selected)) || (selected < 1 || selected > adapterNumber))
            {
                Console.WriteLine("Selected Interface Number is out of range.");
                return;
            }

            //move to selected interface       
            current = head;
            adapter = current.AsStruct<pcap_if>(); //annoying C#...
            for (int i = 0; i < selected-1; ++i)
            {                
                current = adapter.next;
                adapter = current.AsStruct<pcap_if>();
            }
            
            //open the device
            if ((capture = WinPCapDriver.pcap_open_live(adapter.name, 65536, 1, 1000, errbuf)) == nil)
            {
                Console.WriteLine("Capture failed on adapter: {0}", adapter.description);
                WinPCapDriver.pcap_freealldevs(head);
                return;
            }

            //capture some packets!
            Console.WriteLine("Listening on: {0}", adapter.description);

            WinPCapDriver.pcap_loop(capture, 100, new pcap_handler(packet_handler), capture);

            WinPCapDriver.pcap_close(capture);

        }

        /// <summary>
        /// Prints the time the packet was captured and it's length in bytes
        /// </summary>
        /// <param name="user"></param>
        /// <param name="header"></param>
        /// <param name="pkt_data"></param>
        public static void packet_handler(IntPtr user, ref pcap_pkthdr header, IntPtr pkt_data)
        {

            //in Unix, all times are based from Epoch: 1 Jan 1970 at 00:00:00
            DateTime epoch = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);
            DateTime time = epoch.AddSeconds(header.ts.tv_sec);
            time.AddMilliseconds(header.ts.tv_usec * 1000);
            DateTime local = time.ToLocalTime();

            Console.WriteLine("< {0} -> Length: {1} >", local.ToLongTimeString(), header.len);            

        }

    }
}
