﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Net;
using System.Net.Sockets;
using System.Net.NetworkInformation;

using Nomad.Net.PacketCapture.Interop;
using Nomad.Net.PacketCapture.Interop.WinPCap;

namespace iflist
{
    class Program
    {

        public static readonly IntPtr nil = IntPtr.Zero;

        static void Main(string[] args)
        {
            
            //first device in the list
            IntPtr head = IntPtr.Zero;    
            //buffer to hold the error string
            StringBuilder errbuf = new StringBuilder(WinPCapConstants.PCAP_ERRBUF_SIZE);
            //dereference of the device's information
            pcap_if deviceInfo;

            //print the error and exit
            if (WinPCapDriver.pcap_findalldevs(ref head, errbuf) == -1)
            {
                Console.WriteLine("Error in pcap_findalldevs: {0}", errbuf.ToString());
                return;
            }

            //iterate through list and print the device information
            for(IntPtr current = head; current != nil; current = deviceInfo.next)
            {

                deviceInfo = current.AsStruct<pcap_if>();
                ifprint(deviceInfo);

            }

            //clean up the devices
            WinPCapDriver.pcap_freealldevs(head);

            //wait for user to exit
            Console.ReadLine();

        }

        static void ifprint(pcap_if device)
        {

            IntPtr head = device.addresses;
            pcap_addr address;
            sockaddr socketAddress;

            //print basic information
            Console.WriteLine("Device Name: {0}", device.name);
            Console.WriteLine("\tDescription: {0}", device.description);

            Console.WriteLine("\tIs Loopback: {0}", (device.flags & WinPCapConstants.PCAP_IF_LOOPBACK) == 1 ? "yes" : "no");

            for (IntPtr current = head; current != nil; current = address.next)
            {
                                
                address = current.AsStruct<pcap_addr>();
                socketAddress = address.addr.AsStruct<sockaddr>();

                AddressFamily family = (AddressFamily)Enum.Parse(typeof(AddressFamily), socketAddress.sa_family.ToString());
                switch (family)
                {
                        
                    case AddressFamily.InterNetwork:
                        Console.WriteLine("\tAddress Family: AF_INET");

                        //the IP address
                        if(address.addr != nil)
                            Console.WriteLine("\tAddress: {0}", iptos(address.addr.AsStruct<sockaddr_in>().sin_addr.s_addr));

                        //NetMask
                        if (address.netmask != nil)
                            Console.WriteLine("\tNetMask: {0}", iptos(address.netmask.AsStruct<sockaddr_in>().sin_addr.s_addr));

                        //Broadcast Address
                        if (address.broadaddr != nil)
                            Console.WriteLine("\tBroadcast: {0}", iptos(address.broadaddr.AsStruct<sockaddr_in>().sin_addr.s_addr));

                        //Desitination Address
                        if (address.dstaddr != nil)
                            Console.WriteLine("\tDestination: {0}", iptos(address.dstaddr.AsStruct<sockaddr_in>().sin_addr.s_addr));

                        break;

                    case AddressFamily.InterNetworkV6:
                        break;

                    default:
                        Console.WriteLine("\tAddress Family: Unkown");
                        break;

                }

            }

        }

        static string iptos(UInt32 address)
        {

            string addressString = string.Empty;

            byte[] addressBytes = BitConverter.GetBytes(address);

            addressString = string.Format("{0}.{1}.{2}.{3}", addressBytes[0], addressBytes[1], addressBytes[2], addressBytes[3]);

            return addressString;

        }

        static string ip6tos(sockaddr_in address)
        {
            //I have no idea if this works...
            NetworkInterface[] nics = NetworkInterface.GetAllNetworkInterfaces();

            var adapter = from n in nics
                          where n.GetIPProperties().UnicastAddresses[0].Address.ToString() == iptos(address.sin_addr.s_addr)
                          select n;

            if (adapter.Count() >= 0)
                return adapter.ElementAt(0).GetIPProperties().GetIPv6Properties().ToString();

            return string.Empty;

        }

    }

}
