﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Nomad.Net.PacketCapture;
using Nomad.Net.PacketCapture.Interop;
using Nomad.Net.PacketCapture.Interop.WinPCap;
using Nomad.Net.PacketCapture.Interop.Bpf;

namespace pcap_filter
{
    class Program
    {

        public const int MAX_PRINT = 80;
        public const int MAX_LINE = 10;
        public static readonly IntPtr nil = IntPtr.Zero;

        static void Main(string[] args)
        {


            string source = string.Empty; //souce device
            string ofilename = string.Empty; //dump file
            string filter = string.Empty; //filter program
            IntPtr dumpfile;
            StringBuilder errbuf = new StringBuilder(WinPCapConstants.PCAP_ERRBUF_SIZE);
            IntPtr fp;
            bpf_program fcode;
            IntPtr header = nil;
            IntPtr pkt_data = nil;
            UInt32 NetMask;
            int res;

            if (args.Length == 0)
            {
                usage();
                Console.ReadLine();
                return;
            }

            //parse arguments
            for (int i = 0; i < args.Length; ++i)
            {

                switch (args[i])
                {

                    case "-s":
                        source = args[i + 1];
                        break;

                    case "-o":
                        ofilename = args[i + 1];
                        break;

                    case "-f":
                        filter = args[i + 1];
                        break;

                }

            }

            //open a capture from the network
            if (!string.IsNullOrEmpty(source))
            {

                if ((fp = WinPCapDriver.pcap_open_live(source, 65536, 1, 1000, errbuf)) == nil)
                {

                    Console.WriteLine("Unable to open adapter {0}", source);
                    Console.ReadLine();
                    return;

                }

            }
            else
            {
                usage();
                Console.ReadLine();
                return;
            }


            if (!string.IsNullOrEmpty(filter))
            {

                // We should loop through the adapters returned by the pcap_findalldevs_ex()
                // in order to locate the correct one.
                //
                // Let's do things simpler: we suppose to be in a C class network ;-)
                NetMask = 0xffffff;

                if (WinPCapDriver.pcap_compile(fp, out fcode, filter, 1, NetMask) < 0)
                {

                    Console.WriteLine("Error compiling filter: wrong syntax");
                    WinPCapDriver.pcap_close(fp);
                    Console.ReadLine();
                    return;

                }

                //set the filter
                if (WinPCapDriver.pcap_setfilter(fp, ref fcode) < 0)
                {

                    Console.WriteLine("Error setting filter.");
                    WinPCapDriver.pcap_close(fp);
                    Console.ReadLine();
                    return;

                }

            }

            //open the dump file
            if (!string.IsNullOrEmpty(ofilename))
            {

                dumpfile = WinPCapDriver.pcap_dump_open(fp, ofilename);

                if (dumpfile == nil)
                {

                    Console.WriteLine("Error opening output file");
                    WinPCapDriver.pcap_close(fp);
                    Console.ReadLine();
                    return;

                }

            }
            else
            {
                usage();
                Console.ReadLine();
                return;
            }

            //start the capture
            int packetCount = 0;
            while((res = WinPCapDriver.pcap_next_ex(fp, ref header, ref pkt_data)) >= 0) {

                if (res == 0)
                    //timeout expired
                    continue;
                else
                    ++packetCount;

                pcap_pkthdr h = header.AsStruct<pcap_pkthdr>();
                //save packet to dump file
                WinPCapDriver.pcap_dump(dumpfile, ref h, pkt_data);                

                if (packetCount >= MAX_PRINT)
                    break;

                if (packetCount % 10 == 0) Console.WriteLine("Packets: {0}", packetCount);

            }

            Console.WriteLine("Packets: {0}", packetCount);

            WinPCapDriver.pcap_close(fp);
            WinPCapDriver.pcap_dump_close(dumpfile);

            Console.WriteLine("Finished capturing...");
            Console.ReadLine();

        }

        static void usage()
        {
            Console.WriteLine("\npf - Generic Packet Filter.\n");
            Console.WriteLine("\nUsage:\npf -s source -o output_file_name [-f filter_string]\n\n");            
        }

    }

}
